//////////////////////////////////////////////////////////////////////////////
/// \file audio.h
/// Declarations of audio functions
//////////////////////////////////////////////////////////////////////////////

#pragma once

#include <allegro5/allegro_audio.h>

/// Sctructure describing a sound sample
typedef struct {

	/// Is the sample a memory sample
	int  issample;

	/// Is the sample a stream updated from a file
	int  isfilestream;

	/// Is the sample a stream updated by the program
	int  ismemorystream;

	/// Underlying allegro sample object
	ALLEGRO_SAMPLE *sample;

	/// Underlying allegro sample instance object
	ALLEGRO_SAMPLE_INSTANCE *instance;

	/// Underlying allegro stream object
	ALLEGRO_AUDIO_STREAM *stream;

} TS5_SAMPLE;


unsigned int ts5_set_audio_frequency(unsigned int frequency);
unsigned int ts5_get_audio_frequency();

unsigned int ts5_set_audio_channels(unsigned int channels);
unsigned int ts5_get_audio_channels();

int ts5_set_audio_depth(int depth);
int ts5_get_audio_depth();

double ts5_set_audio_gain(double gain);
double ts5_get_audio_gain();


TS5_SAMPLE *ts5_alloc_sample(double length);
TS5_SAMPLE *ts5_read_sample(const char *file);
TS5_SAMPLE *ts5_read_long_sample(const char *file);
void ts5_write_sample(const char *file, TS5_SAMPLE *sample);
void ts5_free_sample(TS5_SAMPLE *sample);

void ts5_play_sample(TS5_SAMPLE *sample);
void ts5_pause_sample(TS5_SAMPLE *sample);
void ts5_stop_sample(TS5_SAMPLE *sample);
int ts5_get_sample_status(TS5_SAMPLE *sample);

// set functies: pass NULL to set preferences for samples that will be made/loaded
unsigned int ts5_get_sample_frequency(TS5_SAMPLE *sample);
unsigned int ts5_get_sample_channels(TS5_SAMPLE *sample);
int ts5_get_sample_depth(TS5_SAMPLE *sample);
unsigned int ts5_get_sample_length(TS5_SAMPLE *sample);
unsigned int ts5_set_sample_position(TS5_SAMPLE *sample, unsigned int position);
unsigned int ts5_get_sample_position(TS5_SAMPLE *sample);

double ts5_set_sample_speed(TS5_SAMPLE *sample, double speed);
double ts5_get_sample_speed(TS5_SAMPLE *sample);

double ts5_set_sample_gain(TS5_SAMPLE *sample, double gain);
double ts5_get_sample_gain(TS5_SAMPLE *sample);

double ts5_set_sample_pan(TS5_SAMPLE *sample, double pan);
double ts5_get_sample_pan(TS5_SAMPLE *sample);

int ts5_set_sample_playmode(TS5_SAMPLE *sample, int playmode);
int ts5_get_sample_playmode(TS5_SAMPLE *sample);

