//////////////////////////////////////////////////////////////////////////////
/// \file audio_unittest.c
/// Unit tests of audio functions
//////////////////////////////////////////////////////////////////////////////

#include "../include/tscope/audio.h"

#include "../include/tscope/system.h"

//////////////////////////////////////////////////////////////////////////////
/// Audio unit tests.
/// Example of:
/// - open the audio subsystem
/// - usage of the parameter functions
/// - alloc and write a sample
/// -
//////////////////////////////////////////////////////////////////////////////
int main(void)
{
	ts5_set_log_level(TS5_LOGLEVEL_5);

	// set some audio parameters
	ts5_set_audio_frequency(TS5_44100);
	ts5_set_audio_channels(TS5_STEREO);
	ts5_set_audio_depth(TS5_FLOAT);
	ts5_set_audio_gain(2.0);

	// install the system & query the settings we got
	printf("got audio frequency: %d\n", ts5_get_audio_frequency());
	printf("got number of channels: %d\n", ts5_get_audio_channels());
	printf("got sample depth: %d\n", ts5_get_audio_depth());
	printf("gain: %f\n", ts5_get_audio_gain());

	// allocate, write & free an sample (silence)
	TS5_SAMPLE *test;
	test = ts5_alloc_sample(1.0);
	ts5_write_sample("test.wav", test);
	ts5_free_sample(test);

	// open a sample from a file, query its format & play
	TS5_SAMPLE *test2;
	test2 = ts5_read_long_sample("test.flac");
	printf("sample frequency: %d\n", ts5_get_sample_frequency(test2));
	printf("sample channels: %d\n", ts5_get_sample_channels(test2));
	printf("sample depth: %d\n", ts5_get_sample_depth(test2));
	printf("sample length: %f\n", ts5_get_sample_length(test2)/(double)ts5_get_sample_frequency(test2));


	printf("play the complete sample\n");
	ts5_play_sample(test2);
	al_rest(5.0);

	printf("play one second\n");
	ts5_play_sample(test2);
	al_rest(1.0);
	printf("pause one second\n");
	ts5_pause_sample(test2);
	al_rest(1.0);
	printf("continue playing\n");
	ts5_play_sample(test2);
	al_rest(3.0);

	printf("play one second\n");
	ts5_play_sample(test2);
	al_rest(1.0);
	printf("stop one second\n");
	ts5_stop_sample(test2);
	al_rest(1.0);
	printf("restart playing\n");
	ts5_play_sample(test2);
	al_rest(3.0);
	ts5_stop_sample(test2);

//	al_rest(3.0);
	printf("set sample position to 0.5 second & play until finished\n");
	ts5_set_sample_position(test2, ts5_get_sample_frequency(test2)*15);
	ts5_play_sample(test2);
//	do {
//		printf("sample position: %d\n", ts5_get_sample_position(test2));
		al_rest(10.0);
//	} while (ts5_get_sample_status(test2));
	printf("finished playing, sample position is now %d\n", ts5_get_sample_position(test2));


	// speed, gain, pan, playmode
	printf("play sample at half speed\n");
	double oldspeed = ts5_set_sample_speed(test2, 0.5);
	ts5_play_sample(test2);
//	do {
		al_rest(10.0);
//	} while (ts5_get_sample_status(test2));
	ts5_set_sample_speed(test2, oldspeed);

	printf("play sample at half gain\n");
	double oldgain = ts5_set_sample_gain(test2, 0.5);
	ts5_play_sample(test2);
//	do {
		al_rest(10.0);
//	} while (ts5_get_sample_status(test2));
	ts5_set_sample_gain(test2, oldgain);


	printf("play sample panned to the left\n");
	double oldpan = ts5_set_sample_pan(test2, -1.0);
	ts5_play_sample(test2);
	al_rest(0.5);
	printf("change pan to right\n");
	ts5_set_sample_pan(test2, 1.0);
	al_rest(0.5);
	printf("change pan to old value\n");
	ts5_set_sample_pan(test2, oldpan);
//	do {
		al_rest(10.0);
//	} while (ts5_get_sample_status(test2));


	printf("play loop\n");
	int oldmode = ts5_set_sample_playmode(test2, TS5_PLAY_LOOP);
	printf("oldmode: %d\n", oldmode);
	ts5_play_sample(test2);
	al_rest(9.0);
	ts5_set_sample_playmode(test2, oldmode);

	ts5_free_sample(test2);

	return 0;
}



