//////////////////////////////////////////////////////////////////////////////
/// \file mouse_unittest.c
/// Unit tests of mouse gui functions
//////////////////////////////////////////////////////////////////////////////

#include "../include/tscope/mouse.h"

#include "../include/tscope/display.h"
#include "../include/tscope/timer.h"
#include "../include/tscope/graphics.h"
#include "../include/tscope/primitives.h"

//////////////////////////////////////////////////////////////////////////////
/// Mouse unit tests.
/// Example of:
/// - changing the position of the mouse on a display
/// - change the display where the mouse is on
/// - wait for a click on a next button
//////////////////////////////////////////////////////////////////////////////
int main(void)
{
	// open two windows
//	ts5_set_log_level(TS5_LOGLEVEL_1);
	double w, h;
	ts5_get_display_adapter_size(0, &w, &h);
/*

	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_display_size(0.25, 0.25);
	ts5_set_display_position(0.5, 0.5);

	ts5_install_display();
	double dw, dh, dx, dy;
	ts5_get_display_size(0, &dw, &dh);
	printf("display size %f %f\n", dw, dh);
	printf("display width %f\n", ts5_get_display_width(1));
	printf("display height %f\n", ts5_get_display_height(1));
	ts5_get_display_position(0, &dx, &dy);
	printf("display position %f %f\n", dx, dy);


	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);


	ts5_draw_line(ts5_relative_to_absolute_coordinate_x(-1.0), 0.0, ts5_relative_to_absolute_coordinate_x(1.0), 0.0);
	ts5_draw_line(0.0, ts5_relative_to_absolute_coordinate_y(-1.0), 0.0, ts5_relative_to_absolute_coordinate_y(1.0));
	
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);

	ts5_set_mouse_position(0.0, 0.0);
	al_rest(1.0);
	ts5_set_mouse_position(0.25, 0.25);
	al_rest(1.0);
	ts5_set_mouse_position(0.5, 0.5);
	al_rest(1.0);
	ts5_draw_mouse_button(0.0, 0.0);
	ts5_wait(1.0);

	double x, y;
	ts5_get_mouse_position(&x, &y);
	printf("mouse position: %f %f\n", x, y);
*/

	
	ts5_set_display_position(w / 4.0, 0.0);
	ts5_install_display();
	ts5_show_mouse();

	// set the mouse position on the display
	ts5_set_mouse_position(0.0, 0.0);
	al_rest(1.0);
	ts5_set_mouse_position(0.0, 100.0);
	al_rest(1.0);
	ts5_set_mouse_position(0.0, -100.0);
	al_rest(1.0);
	ts5_hide_mouse();
	
	// change the display the mouse is on
	ts5_set_active_display(1);
	ts5_set_mouse_position(0.0, 0.0);
	ts5_show_mouse();

	// define the mouse as a response key
	ts5_define_mouse_key(-1);
	ts5_print_status();

	double rt, max=100000.0;
	int resp;
	ts5_flush_responses();
	resp = ts5_wait_for_response_timed(&rt, NULL, max);
	printf("response %d %f\n", resp, rt);
	double x, y;
	ts5_get_mouse_position(&x, &y);
	printf("mouse position after the click: %f %f\n", x, y);
	
	return 0;
}
