//////////////////////////////////////////////////////////////////////////////
/// \file primitives_unittest.c
/// Unit tests of primitives drawing functions
//////////////////////////////////////////////////////////////////////////////

#include "../include/tscope/primitives.h"

#include "../include/tscope/display.h"
#include "../include/tscope/graphics.h"
#include "../include/tscope/mouse.h"

#include <math.h>

//////////////////////////////////////////////////////////////////////////////
/// Primitives functions unit tests.
///
/// Example of:
/// - coordinate systems
/// - color definitions
/// - graphics parameters
/// - fonts and text alignment
//////////////////////////////////////////////////////////////////////////////

double w=600.0, h=600.0;

void wwait()
{
	int oldsystem = ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	int oldscale = ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);	
	ts5_flip_display();
	ts5_draw_mouse_button(0.9, 0.9);
	ts5_clear_display();
	ts5_flip_display();
	ts5_set_coordinate_system(oldsystem);
	ts5_set_coordinate_scale(oldscale);	
}


void pixels_example()
{
	
	double i, x, y;

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=0; i<2*TS5_PI; i+=TS5_PI/32) {
		x = cos(i);
		y = sin(i);
		ts5_draw_pixel(x*100 + w/2, y*100 + h/2);
	}
	wwait();

	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=0; i<2*TS5_PI; i+=TS5_PI/32) {
		x = cos(i);
		y = sin(i);
		ts5_draw_pixel(x*100, y*100);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=0; i<2*TS5_PI; i+=TS5_PI/32) {
		x = cos(i);
		y = sin(i);
		ts5_draw_pixel(x/4 + 0.5, y/4 + 0.5);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=0; i<2*TS5_PI; i+=TS5_PI/32) {
		x = cos(i);
		y = sin(i);
		ts5_draw_pixel(x/2, y/2);
	}
	wwait();
}


void rectangles_example()
{
	double i;

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=1; i<w/2; i+=10) {
		ts5_draw_rectangle(i + w/2, i + h/2, -i + w/2, -i + h/2);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=1; i<w/2; i+=10) {
		ts5_draw_rectangle(i, i, -i, -i);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=0.0; i<0.5; i+=0.02) {
		ts5_draw_rectangle(i + 0.5, i + 0.5, -i + 0.5, -i + 0.5);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=0.0; i<1; i+=0.04) {
		ts5_draw_rectangle(i, i, -i, -i);
	}
	wwait();
}


void rectangles_example2()
{
	double i;
	ts5_set_drawing_thickness(3.0);

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=1; i<=w/2; i+=10) {
		ts5_draw_rectangle(i + w/2.0, i + h/2.0, -i + w/2.0, -i + h/2.0);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=1; i<w/2; i+=10) {
		ts5_draw_rectangle(i, i, -i, -i);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=0.0; i<0.5; i+=0.02) {
		ts5_draw_rectangle(i + 0.5, i + 0.5, -i + 0.5, -i + 0.5);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=0.0; i<1; i+=0.04) {
		ts5_draw_rectangle(i, i, -i, -i);
	}
	wwait();
}


void rectangles_example3()
{
	double i;
	ts5_set_fill_mode(TS5_FILL_ON);

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.1));	
	for (i=1; i<=w/2; i+=10) {
		ts5_draw_rectangle(i + w/2.0, i + h/2.0, -i + w/2.0, -i + h/2.0);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.1));	
	for (i=1; i<w/2; i+=10) {
		ts5_draw_rectangle(i, i, -i, -i);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.1));	
	for (i=0.0; i<0.5; i+=0.02) {
		ts5_draw_rectangle(-i + 0.5, -i + 0.5, i + 0.5, i + 0.5);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.1));	
	for (i=0.0; i<1; i+=0.04) {
		ts5_draw_rectangle(i, i, -i, -i);
	}
	wwait();
}


void rounded_rectangles_example()
{
	double i;
	ts5_set_fill_mode(TS5_FILL_OFF);

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.5));	
	for (i=1; i<=w/2; i+=10) {
		ts5_draw_rounded_rectangle(i + w/2.0, i + h/2.0, -i + w/2.0, -i + h/2.0, 10.0, 10.0);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.5));	
	for (i=1; i<w/2; i+=10) {
		ts5_draw_rounded_rectangle(i, i, -i, -i, 10.0, 10.0);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.5));	
	for (i=0.0; i<0.5; i+=0.02) {
		ts5_draw_rounded_rectangle(-i + 0.5, -i + 0.5, i + 0.5, i + 0.5, 10.0, 10.0);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.5));	
	for (i=0.0; i<1; i+=0.04) {
		ts5_draw_rounded_rectangle(i, i, -i, -i, 10.0, 10.0);
	}
	wwait();
}


void triangles_example()
{
	double i;
	ts5_set_fill_mode(TS5_FILL_OFF);

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.5));	
	for (i=1; i<=w/2; i+=10) {
		ts5_draw_triangle(w/2.0 - i/2, h/2.0 - i, w/2.0 - i/2, h/2.0 + i, w/2 + i, h/2);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.5));	
	for (i=1; i<w/2; i+=10) {
		ts5_draw_triangle(-i/2, - i, -i/2, i, i, 0.0);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.5));	
	for (i=0.0; i<0.5; i+=0.02) {
		ts5_draw_triangle(0.5 - i/2, 0.5 - i, 0.5 - i/2, 0.5 + i, 0.5 + i, 0.5);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.5));	
	for (i=0.0; i<1.0; i+=0.04) {
		ts5_draw_triangle(-i, -i, -i, i, 0.0, 0.0);
	}
	wwait();
}


void circles_example()
{
	double i;

	ts5_set_drawing_thickness(10.0);
	ts5_set_fill_mode(TS5_FILL_OFF);

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.5));	
	for (i=1; i<=w/2; i+=8) {
		ts5_draw_circle(w/2, h/2, i);
	}
	wwait();

	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.5));	
	for (i=1; i<=w/2; i+=8) {
		ts5_draw_circle(0.0, 0.0, i);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.5));	
	for (i=1; i<=w/2; i+=8) {
		ts5_draw_circle(0.5, 0.5, i);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.5));	
	for (i=1; i<=w/2; i+=8) {
		ts5_draw_circle(0.0, 0.0, i);
	}
	wwait();
}


void ellipses_example()
{
	double i;
	ts5_set_fill_mode(TS5_FILL_ON);

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.2));	
	for (i=1; i<=w/2; i+=20) {
		ts5_draw_ellipse(w/2, h/2, i, 100.0);
	}
	wwait();

	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.2));	
	for (i=1; i<=w/2; i+=20) {
		ts5_draw_ellipse(0.0, 0.0, i, 100.0);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 0.2));	
	for (i=1; i<=w/2; i+=20) {
		ts5_draw_ellipse(0.5, 0.5, i, 100.0);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 0.2));	
	for (i=1; i<=w/2; i+=20) {
		ts5_draw_ellipse(0.0, 0.0, i, 100.0);
	}
	wwait();
}


void arc_example()
{
	double i;
	ts5_set_drawing_thickness(5.0);	
	
	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=0; i<=360; i+=45) {
		ts5_draw_arc(w/2, h/2, 100.0, i, 22.5);
	}
	wwait();

	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=0; i<=360; i+=45) {
		ts5_draw_arc(0.0, 0.0, 100.0, i, 22.5);
	}
	wwait();

	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));	
	for (i=0; i<=360; i+=45) {
		ts5_draw_arc(0.5, 0.5, 100.0, i, 22.5);
	}
	wwait();
	
	ts5_set_coordinate_system(TS5_CARTESIAN_COORDINATES);
	ts5_set_coordinate_scale(TS5_RELATIVE_COORDINATES);
	ts5_set_foreground_color(ts5_make_rgb_color(0.0, 1.0, 0.0, 1.0));	
	for (i=0; i<=360; i+=45) {
		ts5_draw_arc(0.0, 0.0, 100.0, i, 22.5);
	}
	wwait();
}


int main(void)
{
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_display_size(w, h);
	ts5_install_display();

	pixels_example();
	rectangles_example();
	rectangles_example2();
	rectangles_example3();
	rounded_rectangles_example();
	triangles_example();
	circles_example();
	ellipses_example();
	arc_example();
	
	return 0;
}
