//////////////////////////////////////////////////////////////////////////////
/// \file randomizer_unittest.c
/// Unit tests of randomizer functions
//////////////////////////////////////////////////////////////////////////////

#include "../include/tscope/randomizer.h"

#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
/// Randomizer unit tests.
/// Example of:
/// - get some uniform doubles
/// - get some uniform integers
/// - get some normal doubles
/// - get some exponential doubles
/// - get a random list without replacement
/// - use the seed to get the same series over and over again
//////////////////////////////////////////////////////////////////////////////
int main(void)
{
	int i;
	printf("some random uniform doubles:\n");
	for (i=0; i<10; i++) {
		printf("%f\n", ts5_random_uniform());
	}
	printf("\n");

	printf("some random uniform integers (max 10):\n");
	for (i=0; i<10; i++) {
		printf("%ld\n", ts5_random_integer(10L));
	}
	printf("\n");

	printf("some random normal doubles (mu=0, sigma=1):\n");
	for (i=0; i<10; i++) {
		printf("%f\n", ts5_random_normal(0.0, 1.0));
	}
	printf("\n");

	printf("some random exponential doubles (mu=1):\n");
	for (i=0; i<10; i++) {
		printf("%f\n", ts5_random_exponential(1.0));
	}
	printf("\n");


	int rnd[6];
	printf("a random list of integers (without replacement):\n");
	ts5_seed_randomizer(1234L);
	ts5_random_list(3, 2, rnd);
	for (i = 0; i < 6; i++) {
		printf("%d %d\n", i, rnd[i]);
	}

	printf("a random list of integers (exactly the same as before):\n");
	ts5_seed_randomizer(1234L);
	ts5_random_list(3, 2, rnd);
	for (i = 0; i < 6; i++) {
		printf("%d %d\n", i, rnd[i]);
	}


	return 0;
}



