//////////////////////////////////////////////////////////////////////////////
/// \file textio_unittest.c
/// Unit tests of text input/output functions
//////////////////////////////////////////////////////////////////////////////

#include "../include/tscope/textio.h"

#include "../include/tscope/display.h"
#include "../include/tscope/graphics.h"
#include "../include/tscope/mouse.h"


int main(void)
{
	ts5_set_coordinate_scale(TS5_ABSOLUTE_COORDINATES);
	ts5_set_display_size(1200.0, 600.0);
//	ts5_set_background_color(ts5_make_rgb_color(1.0, 0.0, 0.0, 1.0));
	ts5_install_display();

	double w, h;
	
	// printf example
	ts5_clear_display();
	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_text_alignment(TS5_ALIGN_LEFT);
	ts5_printf(20.0, 20.0, "an integer: %d", 300);
	ts5_printf(20.0, 40.0, "a double: %08.2lf", 2.0256);
	w = ts5_printf(20.0, 60.0, "a string: %s", "asdf");
	ts5_printf(20.0, 80.0, "width of the last line: %f", w);
	w = ts5_get_text_width("test");
	ts5_printf(20.0, 100.0, "width of \"test\": %f", w);
	h = ts5_get_font_size();
	ts5_printf(20.0, 120.0, "height of this font: %f", h);
		
	ts5_flip_display();
	ts5_draw_mouse_button(ts5_relative_to_absolute_coordinate_x(0.9), ts5_relative_to_absolute_coordinate_y(0.9));

	// printf justify example
	ts5_clear_display();
	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_text_alignment(TS5_ALIGN_LEFT);
	ts5_printf(20.0, 20.0, "justified text asdf asdf asdf");
	ts5_printf_justify(20.0, 420.0, 40.0, 100.0, "justified text asdf asdf asdf");
	ts5_flip_display();
	ts5_draw_mouse_button(ts5_relative_to_absolute_coordinate_x(0.9), ts5_relative_to_absolute_coordinate_y(0.9));

	// printf ustr example
		TS5_USTR *tmp;
		tmp = al_ustr_new("aábdðeéfghiíjklmnoóprstuúvxyýþæö");
	ts5_clear_display();
	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_text_alignment(TS5_ALIGN_LEFT);
	w = ts5_printf_ustr(20.0, 20.0, tmp);
	ts5_printf(20.0, 40.0, "width of the last line: %f", w);
	al_ustr_free(tmp);
	
	ts5_flip_display();
	ts5_draw_mouse_button(ts5_relative_to_absolute_coordinate_x(0.9), ts5_relative_to_absolute_coordinate_y(0.9));
	
	// scanf example
	ts5_clear_display();
	w = ts5_printf(20.0, 20.0, "write something: ");
	ts5_flip_display();
	char buff[TS5_MAX_CHAR];
	ts5_scanf(20+w, 20.0, "%s", buff);
	ts5_printf(20.0, 40.0, "you wrote %s", buff);
	ts5_flip_display();
	ts5_draw_mouse_button(ts5_relative_to_absolute_coordinate_x(0.9), ts5_relative_to_absolute_coordinate_y(0.9));

	// custom font example
	ts5_clear_display();	
	
	TS5_FONT *oldfont, *newfont1, *newfont2, *newfont3;
	newfont1 = ts5_read_font("/usr/local/share/tscope5_data/Courier New Bold.ttf", 20.0);
	oldfont = ts5_set_font(newfont1);
	ts5_printf(40.0, 40.0, "custom font: Courier New Bold 20");
	ts5_set_font(oldfont);

	newfont2 = ts5_read_font ("/usr/local/share/tscope5_data/Arial Bold Italic.ttf", 30.0);
	ts5_set_font(newfont2);
	ts5_printf(40.0, 80.0, "custom font: Arial Bold Italic 30");

	newfont3 = ts5_read_font ("/usr/local/share/tscope5_data/Times New Roman.ttf", 40.0);
	ts5_set_font(newfont3);
	ts5_printf(40.0, 120.0, "custom font: Times New Roman 40");

	ts5_set_font(oldfont);
	ts5_printf(40.0, 160.0, "back to the old font");
	
	ts5_free_font(newfont1);
	ts5_free_font(newfont2);
	ts5_free_font(newfont3);
	
	ts5_flip_display();
	ts5_draw_mouse_button(ts5_relative_to_absolute_coordinate_x(0.9), ts5_relative_to_absolute_coordinate_y(0.9));

/*
	FILE *in;
	in = fopen("unittests/words-pseudowords.txt", "r");
	if (in==NULL) {
		ts5_fatal("error opening file\n");
	}
	char buff1[TS5_MAX_CHAR], buff2[TS5_MAX_CHAR], buff3[TS5_MAX_CHAR];
	
	int i;
	for (i=0; i<11; i++) {
		fscanf(in, "%*s %*s %*s");		
	}
	fscanf(in, "%s %s %s", buff1, buff2, buff3);
	printf("%s %s %s\n", buff1, buff2, buff3);
	ts5_set_font_size(40);
	ts5_printf(0, 50, "%s", buff1);
	ts5_printf(0, 0, "%s", buff2);
	ts5_printf(0, -50, "%s", buff3);
	fclose(in);
	*/
	
	/*
	TS5_USTR *tmp;
		tmp = al_ustr_new("aábdðeéfghiíjklmnoóprstuúvxyýþæö");
	ts5_clear_display();
	ts5_set_coordinate_system(TS5_DISPLAY_COORDINATES);
	ts5_set_text_alignment(TS5_ALIGN_LEFT);
	w = ts5_printf_ustr(20, 20, tmp);
	ts5_printf(20, 40, "width of the last line: %d", w);
	al_ustr_free(tmp);
	
	ts5_flip_display();
	ts5_draw_mouse_button(ts5_relative_to_absolute_coordinate_x(0.9), ts5_relative_to_absolute_coordinate_y(0.9));
  	*/

	return 0;
}
